/*
** ADM-XRC Series Driver
**
** (C) Copyright 2001-2004 Alpha Data
**
** owner.c - IOCTL handlers for taking/releasing ownership of a card.
**
** ----------------------------------------------------------------------------
*/

#include "platform.h"
#include "ioctls.h"
#include "device.h"
#include "owner.h"

static OS_STATUS
grabCard(
  DeviceContext* pDevCtx,
  OS_CLIENT_OBJECT* pClObj,
  os_boolean bShared)
{
  OS_STATUS status = OS_STATUS_SUCCESS;
  ClientContext* pClCtx;
  OS_SPINLOCK_FLAGS f;
  os_boolean bOk;
  os_uint32 val32;

  pClCtx = os_get_client_context(pClObj);
  os_acquire_spinlock(&pDevCtx->owner.lock, &f);
  if (bShared) {
    bOk = (pDevCtx->owner.exclusive == NULL);
  } else {
    bOk = (pDevCtx->owner.exclusive == NULL && !pDevCtx->owner.numShared);
  }
  if (bOk) {
    if (bShared) {
      pClCtx->coopLevel = CoopLevel_Shared;
      pDevCtx->owner.numShared++;
    } else {
      /* Set exclusive client to this client */
      pClCtx->coopLevel = CoopLevel_Exclusive;
      pDevCtx->owner.exclusive = pClObj;

      /* Initialize FPGA space configuration */
      val32 = ADMXRC2_SPACE_SET_WIDTH | ADMXRC2_SPACE_WIDTH_DEFAULT |
	ADMXRC2_SPACE_SET_PREFETCH | ADMXRC2_SPACE_PREFETCH_DEFAULT |
	ADMXRC2_SPACE_SET_BURST | ADMXRC2_SPACE_BURST_DEFAULT;
      pDevCtx->methods.configSpace(pDevCtx, TRUE, 0, &val32);
    }
    os_release_spinlock(&pDevCtx->owner.lock, f);
    os_dbgprint(3, ("grabCard: card grabbed, cardId=%ld bShared=%s client=0x%p\n",
		    (unsigned long) pDevCtx->info.cardInfo.cardId,
		    bShared ? "true" : "false",
		    (void*) pClObj));
  } else {
    os_release_spinlock(&pDevCtx->owner.lock, f);
    status = OS_MAKE_STATUS(ADMXRC2_CARD_BUSY);
    os_dbgprint(3, ("grabCard: card %ld busy\n",
		    (unsigned long) pDevCtx->info.cardInfo.cardId));
  }

  return status;
}

OS_STATUS
ADMXRC_OpenCard2(
  DeviceContext* pDevCtx,
  OS_CLIENT_OBJECT* pClObj,
  OS_IO_REQUEST* pReq,
  ADMXRC_IOCTLS_OPENCARD2* pIoctl,
  unsigned long inSize)
{
  OS_STATUS status;
			
  if (pIoctl == NULL || inSize != sizeof(pIoctl->in)) {
    status = OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }

  status = grabCard(pDevCtx, pClObj, pIoctl->in.bShared);

  return status;
}
